import uuid

from fastapi import Response
from fastapi.datastructures import QueryParams
from sqlmodel import select

from openec2.libvirt import LibvirtSingleton
from openec2.api.describe_instances import (
    InstanceDescription,
    DescribeInstancesResponse,
    DescribeInstancesResponseReservationSet,
    ReservationSet,
    ReservationSetInstancesSet,
    describe_instance,
)
from openec2.db.user import User
from openec2.api.shared import InstanceState
from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.instance import Instance


def describe_instances(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
    user: User,
):
    response_items: list[InstanceDescription] = []
    conn = LibvirtSingleton.of().connection
    for instance in db.exec(select(Instance)).all():
        # Check for permission issues
        if instance.owner_id != user.id:
            # TODO: Add the error to the response
            continue

        dom = conn.lookupByName(instance.id)
        response_items.append(
            describe_instance(instance, dom),
        )

    return Response(
        DescribeInstancesResponse(
            request_id=uuid.uuid4().hex,
            reservation_set=DescribeInstancesResponseReservationSet(
                item=[
                    ReservationSet(
                        reservationId=instance.instance_id,
                        instancesSet=ReservationSetInstancesSet(
                            item=[instance],
                        ),
                    )
                    for instance in response_items
                ],
            ),
        ).to_xml(),
        media_type="application/xml",
    )
