from sqlmodel import SQLModel, Field, PrimaryKeyConstraint

from openec2.utils.ip import int_to_ipv4, ipv4_to_int


class IPAMEntry(SQLModel, table=True):
    # IP Address
    ipv4_addr_raw: int = Field(primary_key=True)

    # Instance this IP is assigned to
    instance_id: str = Field(primary_key=True)

    # VPC ID
    vpc_id: str

    def ipv4(self) -> str:
        return int_to_ipv4(self.ipv4_addr_raw)

    def set_ipv4(self, addr: str):
        self.ipv4_addr_raw = ipv4_to_int(addr)

    __table_args = (PrimaryKeyConstraint("ipv4_addr_raw", "vpc_id"),)
